# Issues reported upstream
# Python libraries are not installed in the right location
# https://github.com/INCF/libneurosim/issues/12
# PR opened here:
# https://github.com/INCF/libneurosim/pull/14

# Obsolete autotools m4 used
# https://github.com/INCF/libneurosim/issues/11

# %%global commit 57b76e222dd3196fa3c97ddfa2f8132627814c23
%global commit 7d074da5124d6d2db4fd129706fa8465f99ffe9f
%global shortcommit %(c=%{commit}; echo ${c:0:7})

%global _description \
libneurosim is a general library that provides interfaces and common utility \
code for neuronal simulators. \
\
Currently it provides the ConnectionGenerator interface. \
\
The ConnectionGenerator API is a standard interface supporting efficient \
generation of network connectivity during model setup in neuronal network \
simulators. It is intended as an abstraction isolating both sides of the API: \
any simulator can use a given connection generator and a given simulator can \
use any library providing the ConnectionGenerator interface. It was initially \
developed to support the use of libcsa from NEST.

%global with_mpich 1
%global with_openmpi 1

# https://fedoraproject.org/wiki/Packaging:DistTag?rd=Packaging/DistTag#Conditionals
# Disabled for rawhide + F30
%if 0%{?fedora} < 30
%global with_py2 1
%else
%global with_py2 0
%endif

Name:           libneurosim
Version:        0
Release:        2.20181028.git%{shortcommit}%{?dist}
Summary:        Common interfaces for neuronal simulators

License:        GPLv3+
URL:            https://github.com/INCF/%{name}
# Source0:        https://github.com/INCF/%%{name}/archive/%%{commit}/%%{name}-%%{shortcommit}.tar.gz
Source0:        https://github.com/sanjayankur31/%{name}/archive/%{commit}/%{name}-%{shortcommit}.tar.gz

BuildRequires:  automake
BuildRequires:  autoconf
BuildRequires:  libtool-ltdl-devel
BuildRequires:  libtool
BuildRequires:  gcc-c++
BuildRequires:  python3-devel

%if %{with_py2}
BuildRequires:  python2-devel
%endif

# Pull in the common package
Requires:       %{name}-common = %{version}-%{release}

%description
%{_description}

%package        devel
Summary:        Development files for %{name}
Requires:       %{name}%{?_isa} = %{version}-%{release}

%description    devel
The %{name}-devel package contains libraries and header files for
developing applications that use %{name}.

%package        common
Summary:        Common files for %{name}
BuildArch:      noarch

%description    common
The %{name}-common package contains files required by all sub packages.

%package -n python3-%{name}
Summary:        Python3 support for %{name}
Requires:       %{name}-common = %{version}-%{release}
%{?python_provide:%python_provide python3-%{name}}

%description -n python3-%{name}
%{_description}

%if %{with_py2}
%package -n python2-%{name}
Summary:        Python2 support for %{name}
Requires:       %{name}-common = %{version}-%{release}
%{?python_provide:%python_provide python2-%{name}}

%description -n python2-%{name}
%{_description}
%endif

%if %{with_openmpi}
%package openmpi
Summary:        %{name} built with openmpi
BuildRequires:  openmpi-devel
BuildRequires:  rpm-mpi-hooks
Requires:       openmpi
Requires:       %{name}-common = %{version}-%{release}
%description openmpi
%{_description}

%package openmpi-devel
Summary:        %{name} built with openmpi
BuildRequires:  openmpi-devel
BuildRequires:  rpm-mpi-hooks
Requires:       openmpi
Requires:       %{name}-openmpi%{?_isa} = %{version}-%{release}
%description openmpi-devel
%{_description}

%package -n python3-%{name}-openmpi
Summary:        Python3 support for %{name} built with openmpi
BuildRequires:  openmpi-devel
BuildRequires:  rpm-mpi-hooks
Requires:       openmpi
Requires:       %{name}-common = %{version}-%{release}
%{?python_provide:%python_provide python3-%{name}-openmpi}

%description -n python3-%{name}-openmpi
%{_description}

%if %{with_py2}
%package -n python2-%{name}-openmpi
Summary:        Python2 support for %{name} built with openmpi
BuildRequires:  openmpi-devel
BuildRequires:  rpm-mpi-hooks
Requires:       openmpi
Requires:       %{name}-common = %{version}-%{release}
%{?python_provide:%python_provide python2-%{name}-openmpi}

%description -n python2-%{name}-openmpi
%{_description}
%endif
%endif


%if %{with_mpich}
%package mpich
Summary:        %{name} built with mpich
BuildRequires:  mpich-devel
BuildRequires:  rpm-mpi-hooks
Requires:       mpich
Requires:       %{name}-common = %{version}-%{release}

%description mpich
%{_description}

%package mpich-devel
Summary:        %{name} built with mpich
BuildRequires:  mpich-devel
BuildRequires:  rpm-mpi-hooks
Requires:       mpich
Requires:       %{name}-mpich%{?_isa} = %{version}-%{release}
%description mpich-devel
%{_description}

%package -n python3-%{name}-mpich
Summary:        Python3 support for %{name} built with mpich
BuildRequires:  mpich-devel
BuildRequires:  rpm-mpi-hooks
Requires:       mpich
Requires:       %{name}-common = %{version}-%{release}
%{?python_provide:%python_provide python3-%{name}-mpich}

%description -n python3-%{name}-mpich
%{_description}

%if %{with_py2}
%package -n python2-%{name}-mpich
Summary:        Python2 support for %{name} built with mpich
BuildRequires:  mpich-devel
BuildRequires:  rpm-mpi-hooks
Requires:       mpich
Requires:       %{name}-common = %{version}-%{release}
%{?python_provide:%python_provide python2-%{name}-mpich}

%description -n python2-%{name}-mpich
%{_description}
%endif
%endif

%prep
%autosetup -c -n %{name}-%{commit}

# Make these accessible here
cp -v %{name}-%{commit}/COPYING .
cp -v %{name}-%{commit}/README.md .

# Default is py3
%if %{with_py2}
    cp -a %{name}-%{commit} %{name}-%{commit}-py2
%endif

%if %{with_mpich}
    %if %{with_py2}
        cp -a %{name}-%{commit}-py2 %{name}-%{commit}-mpich-py2
    %endif
    cp -a %{name}-%{commit} %{name}-%{commit}-mpich
%endif

%if %{with_openmpi}
    %if %{with_py2}
        cp -a %{name}-%{commit}-py2 %{name}-%{commit}-openmpi-py2
    %endif
    cp -a %{name}-%{commit} %{name}-%{commit}-openmpi
%endif

%build

%global do_build \
pushd %{name}-%{commit}$MPI_COMPILE_TYPE && \
./autogen.sh && \
%{set_build_flags} \
./configure --disable-static \\\
--disable-silent-rules \\\
--with-python=$PYTHON_VERSION \\\
--with-mpi=$MPI_YES \\\
--prefix=$MPI_HOME \\\
--libdir=$MPI_LIB \\\
--includedir=$MPI_INCLUDE \\\
--bindir=$MPI_BIN \\\
--mandir=$MPI_MAN && \
%make_build STRIP=/bin/true && \
popd || exit -1


# Python 3
MPI_COMPILE_TYPE=""
PYTHON_VERSION=3
MPI_YES="no"
MPI_HOME=%{_prefix}
MPI_LIB=%{_libdir}
MPI_INCLUDE=%{_includedir}
MPI_BIN=%{_bindir}
MPI_MAN=%{_mandir}
%{do_build}

# Python 2
%if %{with_py2}
MPI_COMPILE_TYPE="-py2"
PYTHON_VERSION=2
MPI_YES="no"
MPI_HOME=%{_prefix}
MPI_LIB=%{_libdir}
MPI_INCLUDE=%{_includedir}
MPI_BIN=%{_bindir}
MPI_MAN=%{_mandir}
%{do_build}
%endif

# Mpich
%if %{with_mpich}
%{_mpich_load}
# Python 3
MPI_COMPILE_TYPE="-mpich"
PYTHON_VERSION=3
MPI_YES="yes"
%{do_build}

# Python 2
%if %{with_py2}
MPI_COMPILE_TYPE="-mpich-py2"
PYTHON_VERSION=2
MPI_YES="yes"
%{do_build}
%endif
%{_mpich_unload}
%endif

# Openmpi
%if %{with_openmpi}
%{_openmpi_load}
# Python 3
MPI_COMPILE_TYPE="-openmpi"
PYTHON_VERSION=3
MPI_YES="yes"
%{do_build}

# Python 2
%if %{with_py2}
MPI_COMPILE_TYPE="-openmpi-py2"
PYTHON_VERSION=2
MPI_YES="yes"
%{do_build}
%endif
%{_openmpi_unload}
%endif

%install
%global do_install \
%make_install -C %{name}-%{commit}$MPI_COMPILE_TYPE STRIP=/bin/true || exit -1

# Move the python mpi files to the right place
%global correct_python_loc \
mkdir $RPM_BUILD_ROOT/$PYTHON_LOC -v \
mv $RPM_BUILD_ROOT/%{_libdir}/$MPI_TYPE/%{_lib}/python$PY_VERSION/site-packages/libpy*neurosim.so $RPM_BUILD_ROOT/$PYTHON_LOC/ -v


# Python 3
MPI_COMPILE_TYPE=""
PYTHON_VERSION=3
%{do_install}

# Python 2
%if %{with_py2}
MPI_COMPILE_TYPE="-py2"
PYTHON_VERSION=2
%{do_install}
%endif

# Mpich
%if %{with_mpich}
%{_mpich_load}
# Python 3
MPI_TYPE="mpich"
MPI_COMPILE_TYPE="-mpich"
PYTHON_VERSION=3
PY_VERSION=%{python3_version}
PYTHON_LOC=$MPI_PYTHON3_SITEARCH
%{do_install}
%{correct_python_loc}

# Python 2
%if %{with_py2}
MPI_COMPILE_TYPE="-mpich-py2"
PYTHON_VERSION=2
PY_VERSION=%{python2_version}
PYTHON_LOC=$MPI_PYTHON2_SITEARCH
%{do_install}
%{correct_python_loc}
%endif
%{_mpich_unload}
%endif

# Openmpi
%if %{with_openmpi}
%{_openmpi_load}
MPI_TYPE="openmpi"
# Python3
MPI_COMPILE_TYPE="-openmpi"
PYTHON_VERSION=3
PY_VERSION=%{python3_version}
PYTHON_LOC=$MPI_PYTHON3_SITEARCH
%{do_install}
%{correct_python_loc}

# Python 2
%if %{with_py2}
MPI_COMPILE_TYPE="-openmpi-py2"
PYTHON_VERSION=2
PY_VERSION=%{python2_version}
PYTHON_LOC=$MPI_PYTHON2_SITEARCH
%{do_install}
%{correct_python_loc}
%endif
%{_openmpi_unload}
%endif

find $RPM_BUILD_ROOT -name '*.la' -exec rm -f {} ';'

%ldconfig_scriptlets

%files
%{_libdir}/%{name}.so.0
%{_libdir}/%{name}.so.0.0.0

%files devel
%{_includedir}/neurosim
%{_libdir}/*.so

%files common
%license COPYING
%doc README.md

%files -n python3-%{name}
%{python3_sitearch}/libpy3neurosim.so

%if %{with_py2}
%files -n python2-%{name}
%{python2_sitearch}/libpy2neurosim.so
%{python2_sitearch}/libpyneurosim.so
%endif

%if %{with_mpich}
%files mpich
%{_libdir}/mpich/lib/%{name}.so.0
%{_libdir}/mpich/lib/%{name}.so.0.0.0

%files mpich-devel
%{_includedir}/mpich*/neurosim
%{_libdir}/mpich/lib/*.so

%files -n python3-%{name}-mpich
%{python3_sitearch}/mpich/libpy3neurosim.so

%if %{with_py2}
%files -n python2-%{name}-mpich
%{python2_sitearch}/mpich/libpy2neurosim.so
%{python2_sitearch}/mpich/libpyneurosim.so
%endif
%endif

%if %{with_openmpi}
%files openmpi
%{_libdir}/openmpi/lib/%{name}.so.0
%{_libdir}/openmpi/lib/%{name}.so.0.0.0

%files openmpi-devel
%{_includedir}/openmpi*/neurosim
%{_libdir}/openmpi/lib/*.so

%files -n python3-%{name}-openmpi
%{python3_sitearch}/openmpi/libpy3neurosim.so

%if %{with_py2}
%files -n python2-%{name}-openmpi
%{python2_sitearch}/openmpi/libpy2neurosim.so
%{python2_sitearch}/openmpi/libpyneurosim.so
%endif
%endif

%changelog
* Sun Oct 28 2018 Ankur Sinha <ankursinha AT fedoraproject DOT org> - 0-2.20181028.git7d074da
- Rebuild using conditional

* Thu Oct 25 2018 Ankur Sinha <ankursinha AT fedoraproject DOT org> - 0-1.20181025.git7d074da
- Place python so in correct location
- Correct devel file list

* Fri Oct 19 2018 Ankur Sinha <ankursinha AT fedoraproject DOT org> - 0-1.20181019.git57b76e2
- Correct release field
- Correct autosetup usage
- Move common files to -common sub package
- Explicitly version sonames
- Use tweaks suggested in review
- Make python3 default
- Enable debuginfo
- Update to latest upstream commit
- Initial build
